#ifndef CHROMEOS_CAMERA_LIB_AUTO_FRAMING_AUTO_FRAMING_CROS_H_
#define CHROMEOS_CAMERA_LIB_AUTO_FRAMING_AUTO_FRAMING_CROS_H_

#include <stdint.h>

#include <memory>
#include <string>

namespace cros {

// Auto-framing pipeline interface for portable ChromeOS library.
class AutoFramingCrOS {
 public:
  static constexpr double kMinFrameRate = 1.0;
  static constexpr double kMaxFrameRate = 1000.0;
  static constexpr int kMinImageDimension = 100;
  static constexpr int kMaxImageDimension = 10000;
  static constexpr int kMaxAspectRatio = 100;

  // Image formats that map to mediapipe::ImageFormat::Format.
  enum class ImageFormat {
    kSRGB,
    kGRAY8,
  };

  // AutoFramingCrOS config options.
  struct Options {
    // Source video parameters.
    double frame_rate = 0.0;
    int image_width = 0;
    int image_height = 0;

    // Parameters of the intermediate frame to run detection on.
    ImageFormat detector_input_format = ImageFormat::kGRAY8;
    int detector_input_width = 0;
    int detector_input_height = 0;

    // Aspect ratio of the output crop window.
    int target_aspect_ratio_x = 0;
    int target_aspect_ratio_y = 0;
  };

  // Callbacks for receiving auto-framing pipeline processed results.
  class Client {
   public:
    virtual ~Client() = default;

    // Called the when the frame sent by `ProcessFrame()` is processed and can
    // be released.
    virtual void OnFrameProcessed(int64_t timestamp) = 0;

    // Called when a region of interest (ROI) is calculated.  ROI contains the
    // detected saliency points of the single frame without motion model
    // applied. Depending on the detection speed, this is not called on every
    // frame that passed with `ProcessFrame()`.
    virtual void OnNewRegionOfInterest(int64_t timestamp, int x_min, int y_min,
                                       int x_max, int y_max) = 0;
    // Called when a crop rectangle is calculcated.  Motion model is applied so
    // it should move smoothly.
    virtual void OnNewCropWindow(int64_t timestamp, int x_min, int y_min,
                                 int x_max, int y_max) = 0;
  };

  // Factory method to create an AutoFramingCrOS instance.
  static std::unique_ptr<AutoFramingCrOS> Create();

  virtual ~AutoFramingCrOS() = default;

  // Initialize with `options` and register callbacks with `client`.  Override
  // the built-in graph config with `graph_config_override` if not null.
  virtual bool Initialize(const Options& options, Client* client,
                          const std::string* graph_config_override) = 0;

  // Notify there's an incoming video frame at `timestamp`.  The pipeline will
  // generate results at the `timestamp` via client callbacks.
  virtual bool NotifyFrame(int64_t timestamp) = 0;

  // Queue an image to run detection on.  `data` contains image pixels in the
  // format specified with `Options::detector_input_*`.  `timestamp` is
  // carried to the Client callbacks.  `data` should live long enough until
  // the client received `OnFrameProcessed()` with the same `timestamp`.
  virtual bool ProcessFrame(int64_t timestamp, const uint8_t* data,
                            int stride) = 0;

  // Move the target crop window to the specified coordinates regardless of
  // previous detection results.
  virtual bool SetTargetCropWindow(int64_t timestamp, int x_min, int y_min,
                                   int x_max, int y_max) = 0;

  // Wait until all the queued frames are processed.
  virtual bool Wait() = 0;
};

}  // namespace cros

#endif  // CHROMEOS_CAMERA_LIB_AUTO_FRAMING_AUTO_FRAMING_CROS_H_
